#ifndef _SsaoShader_H__
#define _SsaoShader_H__

#include "common.h"
#include "glInterface.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>


namespace CHCDemoEngine 
{ 

class FrameBufferObject;
class Vector3;
class Camera;


/** This class implements a deferred shading algorithm that takes
	a frame buffer object as input and outputs an image in the given size
*/
class SsaoShader
{
public:
	/** constructor for a deferred shader taking the requested output image size,
		the current camera, the exponential smoothing factor for temporal reprojection, 
		and a scaling factor.
			
		The parameter scaleFactor must be reciprocal value of the 
		scale factor used for creating the positions texture. It is used recover the 
		exact scene size that was scaled in order to improve floating point precision.
	*/
	SsaoShader(int w, int h, Camera *cam, float expFactor, float scaleFactor);
	/** The algorithm renders the scene given an fbo.
		The fbo must have color buffer, position buffer, normal buffer.
	*/
	void Render(FrameBufferObject *fbo, FrameBufferObject *fbo2);

	/** Initialises the deferred shader and loads the required shaders:
		This function has to be called only once.
	*/
	static void Init(CGcontext context);


protected:

	void FirstPass(FrameBufferObject *fbo, FrameBufferObject *fbo2);

	void SecondPass(FrameBufferObject *fbo, FrameBufferObject *fbo2);


	////////////

	int mWidth;
	int mHeight;

	/// the temporal smoothing factor
	float mExpFactor;
	/// this is just a scale factor for the scene depth in order to get better float precision in the shader
	float mScaleFactor;

	Camera *mCamera;


private:

	void CreateNoiseTex2D();
	void ComputeViewVectors(Vector3 &tl, Vector3 &tr, Vector3 &bl, Vector3 &br);

};

} // namespace 
#endif // _SsaoShader_H__